#========================================================================================
# package      DE_EPAGES::Spreedly::API::PurchaseController
# state        public
# description  Purchase controller
#========================================================================================
package DE_EPAGES::Spreedly::API::PurchaseController;

use DE_EPAGES::Spreedly::API::RestClient;

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Spreedly::API::PurchaseController->new( $Servlet );
#----------------------------------------------------------------------------------------
# description  instantiate controller to manage purchase tasks.
#---------------------------- ------------------------------------------------------------
# return       $GatewayController | Gateway controller | DE_EPAGES::Spreedly::API::GatewayController
#========================================================================================
sub new {
    my $class = shift;

    my $self = bless {}, $class;

    return $self;
}

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Spreedly::API::PurchaseController->new( $Servlet );
#----------------------------------------------------------------------------------------
# description  instantiate controller to manage purchase tasks.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
# return       $GatewayController | gateway controller | DE_EPAGES::Spreedly::API::GatewayController
#========================================================================================
sub makePurchase {
    my $class = shift;
    my ( $LineItemContainer, $Shop, $GatewayName, $SpreedlyBrowserInfo) = @_;

    my $RestClient = DE_EPAGES::Spreedly::API::RestClient->create( { Shop => $Shop } );
    my $hTransaction = $RestClient->makePurchase( $LineItemContainer, $GatewayName, $SpreedlyBrowserInfo);

    return $hTransaction;
}

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Spreedly::API::PurchaseController->new( $Servlet );
#----------------------------------------------------------------------------------------
# description  Make complete request.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
# return       $hTransaction | Spreedly transaction response
#========================================================================================
sub CompleteTransaction {
    my $class = shift;
    my ($Shop, $GatewayName, $TransactionToken) = @_;

    my $RestClient = DE_EPAGES::Spreedly::API::RestClient->create( { Shop => $Shop } );
    my $hTransaction = $RestClient->CompleteTransaction($GatewayName, $TransactionToken);

    return $hTransaction;
}

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Spreedly::API::PurchaseController->new( $Servlet );
#----------------------------------------------------------------------------------------
# description  Make show transaction request.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
# return       $hTransaction | Spreedly transaction response
#========================================================================================
sub ShowTransaction {
    my $class = shift;
    my ($Shop, $GatewayName, $TransactionToken) = @_;

    my $RestClient = DE_EPAGES::Spreedly::API::RestClient->create( { Shop => $Shop } );
    my $hTransaction = $RestClient->ShowTransaction($GatewayName, $TransactionToken);

    return $hTransaction;
}


1;
