#==============================================================================
# package      DE_EPAGES::Spreedly::Hooks::OrderProcess
# state        private
#==============================================================================
package DE_EPAGES::Spreedly::Hooks::OrderProcess;

use DE_EPAGES::Spreedly::UI::BasketOrder;

use strict;

#========================================================================================
# function     AfterSaveMultiCheckoutPayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       AfterSaveMultiCheckoutPayment($hParams);
#----------------------------------------------------------------------------------------
# description  gather spreedly's payment information token
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub AfterSaveMultiCheckoutPayment {
    my ($hParams) = @_;

    my $Servlet           = $hParams->{'Servlet'};
    my $Basket            = $Servlet->object();
    my $LineItemContainer = $Basket->container();
    my $Payment           = $LineItemContainer->get('Payment');

    # we want to run this hook only when transitioning from payment to confirmation, not when displaying payment
    return if ( $Servlet->params('ViewAction') eq 'ViewMultiCheckoutPayment' );
    return unless ( defined($Payment) && $Payment->instanceOf('LineItemPaymentSpreedly') );

    DE_EPAGES::Spreedly::UI::BasketOrder->RetrievePaymentInfoToken($Servlet);

    return;
}

#========================================================================================
# function     OnOrderCreatedMultiStepCheckout
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnOrderCreatedMultiStepCheckout($hParams);
#----------------------------------------------------------------------------------------
# description  Set Order to paid
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Order - order - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnOrderCreatedMultiStepCheckout {
    my ($hParams) = @_;

    my $Order = $hParams->{'Order'};
    return unless defined($Order);

    my $LineItemContainer = $Order->container();
    return unless defined($LineItemContainer);

    my $LineItemPayment = $LineItemContainer->get('Payment');
    return unless ( defined($LineItemPayment) && $LineItemPayment->instanceOf('LineItemPaymentSpreedly') );

    $LineItemPayment->setOrderToPaid();

    return;
}

#========================================================================================
# function     OnSaveMultiCheckoutConfirmation
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnSaveMultiCheckoutConfirmation($hParams);
#----------------------------------------------------------------------------------------
# description  trigger ConfirmSuccessfulTransaction and confirm spreedly's response
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnSaveMultiCheckoutConfirmation {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $Basket = $Servlet->object();
    my $LineItemContainer = $Basket->container();
    my $LineItemPayment = $LineItemContainer->get('Payment');

    return unless ( defined($LineItemPayment) && $LineItemPayment->instanceOf('LineItemPaymentSpreedly') );
    DE_EPAGES::Spreedly::UI::BasketOrder->ConfirmSuccessfulTransaction($Servlet);

    return;
}

1;
