#========================================================================================
# package      DE_EPAGES::Spreedly::t::unit::UI_JSON
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for UI::JSON
#========================================================================================
package DE_EPAGES::Spreedly::t::unit::UI_JSON;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Spreedly::UI::JSON;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall WasCalled );
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::Exception;
use Test::MockObject::Extends;
use Test::More;
use DE_EPAGES::Spreedly::API::Constants qw ( 
    PURCHASE_RESPONSE_STATE_SUCEEDED
    PURCHASE_RESPONSE_STATE_PENDING
    JSON_FAILURE_MESSAGES );

use strict;

use constant PAYMENT_INFO_TOKEN => 'EWdi0wRvmXUX5pOqPgvhTUtV8g3';

sub test {
    my $self = shift;

    $self->_test_MakePurchase_SuccessfulTransaction_SuccessState();
    $self->_test_MakePurchase_SuccessfulTransaction_PendingState_ThreeDSAvailable();
    $self->_test_MakePurchase_SuccessfulTransaction_PendingState_ThreeDSNotAvailable();
    $self->_test_MakePurchase_SuccessfulTransaction_ErrorState();
    $self->_test_MakePurchase_UnsuccessfulTransaction();
    $self->_test_MakePurchase_WithError();
    $self->_test_CompleteTransaction_SuccessfulTransaction_SuccessState_ThreeDSAvailable();
    $self->_test_CompleteTransaction_SuccessfulTransaction_PendingState_ThreeDSAvailable();
    $self->_test_CompleteTransaction_SuccessfulTransaction_PendingState_ThreeDSNotAvailable();
    $self->_test_CompleteTransaction_SuccessfulTransaction_ErrorState();
    $self->_test_CompleteTransaction_UnsuccessfulTransaction();
    $self->_test_CompleteTransaction_WithError();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub _test_MakePurchase_SuccessfulTransaction_SuccessState {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUIBasketOrder = $self->_buildUIBasketOrder_SuccessfulTransactionSuccessState();
    my $MockedServlet       = $self->_buildServlet();


    $MockedUIBasketOrder->MakePurchase($MockedServlet);
    ok( WasCalled( $MockedUIBasketOrder, '_SaveTransactionNumber' ), "$SubTestName - _SaveTransactionNumber has been called." );
    ok( WasCalled( $MockedUIBasketOrder, '_writeDataToOutput' ), "$SubTestName - _writeDataToOutput has been called." );
    ok( WasCalled( $MockedUIBasketOrder, '_SaveTransactionToken' ), "$SubTestName - _SaveTransactionToken has been called." );

    my $hExpectedTransaction = $self->_getSuccessfulTransaction(); 

    my $aExpectedSaveTransactionParameters = [$hExpectedTransaction, $MockedServlet->object()];
    my $aActualSaveTransactionParameters = GetParametersFromMockifyCall($MockedUIBasketOrder, '_SaveTransactionNumber');
    is_deeply($aActualSaveTransactionParameters, $aExpectedSaveTransactionParameters ,"$SubTestName - _SaveTransactionNumber has correct parameters." );

    my $aExpectedResponse = [{'Transaction' => {
        'state' => $hExpectedTransaction->{'state'},
        'required_action' => $hExpectedTransaction->{'required_action'},
    }}];
    my $aActualResponse = GetParametersFromMockifyCall($MockedUIBasketOrder, '_writeDataToOutput');
    is_deeply($aActualResponse, $aExpectedResponse ,"$SubTestName - _writeDataToOutput has correct parameters." );

    return;
}

sub _test_MakePurchase_SuccessfulTransaction_PendingState_ThreeDSAvailable {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsThreeDSAvailable = 1;
    my $MockedUIBasketOrder = $self->_buildUIBasketOrder_SuccessfulTransactionPendingState($IsThreeDSAvailable);
    my $MockedServlet       = $self->_buildServlet();
    $MockedUIBasketOrder->MakePurchase($MockedServlet);
    ok( WasCalled( $MockedUIBasketOrder, '_HandleSuccessfulPurchaseResponse' ), "$SubTestName - _HandleSuccessfulPurchaseResponse has been called." );
    ok( WasCalled( $MockedUIBasketOrder, '_writeDataToOutput' ), "$SubTestName - _writeDataToOutput has been called." );
    ok( !WasCalled( $MockedUIBasketOrder, '_SaveTransactionNumber' ), "$SubTestName - _SaveTransactionNumber has not been called." );
    ok( WasCalled( $MockedUIBasketOrder, '_SaveTransactionToken' ), "$SubTestName - _SaveTransactionToken has been called." );

    my $hExpectedTransaction = $self->_getPendingTransaction();

    my $aExpectedResponse = [{'Transaction' => {
            'state' => $hExpectedTransaction->{'state'},
            'token' => $hExpectedTransaction->{'token'},
            'required_action' => $hExpectedTransaction->{'required_action'},
            'device_fingerprint_form' => $hExpectedTransaction->{'device_fingerprint_form'},
            'checkout_form' => $hExpectedTransaction->{'checkout_form'},
            'checkout_url' => $hExpectedTransaction->{'checkout_url'},
    }}];
    my $aActualResponse = GetParametersFromMockifyCall($MockedUIBasketOrder, '_writeDataToOutput');
    is_deeply($aActualResponse, $aExpectedResponse ,"$SubTestName - _writeDataToOutput has correct parameters." );

    return;
}

sub _test_MakePurchase_SuccessfulTransaction_PendingState_ThreeDSNotAvailable {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsThreeDSAvailable = 0;
    my $MockedUIBasketOrder = $self->_buildUIBasketOrder_SuccessfulTransactionPendingState($IsThreeDSAvailable);
    my $MockedServlet       = $self->_buildServlet();
    $MockedUIBasketOrder->MakePurchase($MockedServlet);
    ok( WasCalled( $MockedUIBasketOrder, '_HandleSuccessfulPurchaseResponse' ), "$SubTestName - _HandleSuccessfulPurchaseResponse has been called." );
    ok( WasCalled( $MockedUIBasketOrder, '_writeDataToOutput' ), "$SubTestName - _writeDataToOutput has been called." );
    ok( !WasCalled( $MockedUIBasketOrder, '_SaveTransactionNumber' ), "$SubTestName - _SaveTransactionNumber has not been called." );
    ok( WasCalled( $MockedUIBasketOrder, '_SaveTransactionToken' ), "$SubTestName - _SaveTransactionToken has been called." );

    my $aExpectedResponse = [{
        'Error' => {
                'Name' => JSON_FAILURE_MESSAGES->{'Name'}->{'Purchase'},
                'Reason' => JSON_FAILURE_MESSAGES->{'Reason'}->{'CardDeclined'}
        }
    }];

    my $aActualResponse = GetParametersFromMockifyCall($MockedUIBasketOrder, '_writeDataToOutput');
    is_deeply($aActualResponse, $aExpectedResponse ,"$SubTestName - _writeDataToOutput has correct parameters." );

    return;
}

sub _test_MakePurchase_SuccessfulTransaction_ErrorState {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUIBasketOrder = $self->_buildUIBasketOrder_SuccessfulTransactionErrorState();
    my $MockedServlet       = $self->_buildServlet();
    $MockedUIBasketOrder->MakePurchase($MockedServlet);
    ok( WasCalled( $MockedUIBasketOrder, '_HandleSuccessfulPurchaseResponse' ), "$SubTestName - _HandleSuccessfulPurchaseResponse has been called." );
    ok( WasCalled( $MockedUIBasketOrder, '_writeDataToOutput' ), "$SubTestName - _writeDataToOutput has been called." );
    ok( WasCalled( $MockedUIBasketOrder, '_SaveTransactionToken' ), "$SubTestName - _SaveTransactionToken has not been called." );
    ok( !WasCalled( $MockedUIBasketOrder, '_SaveTransactionNumber' ), "$SubTestName - _SaveTransactionNumber has not been called." );

    my $aExpectedResponse = [{
        'Error' => {
                'Name' => JSON_FAILURE_MESSAGES->{'Name'}->{'Purchase'},
                'Reason' => JSON_FAILURE_MESSAGES->{'Reason'}->{'CardDeclined'}
        }
    }];
    my $aActualResponse = GetParametersFromMockifyCall($MockedUIBasketOrder, '_writeDataToOutput');
    is_deeply($aActualResponse, $aExpectedResponse ,"$SubTestName - _writeDataToOutput has correct parameters." );

    return;
}

sub _test_MakePurchase_UnsuccessfulTransaction {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUIBasketOrder = $self->_buildUIBasketOrder_UnsuccessfulTransaction();
    my $MockedServlet       = $self->_buildServlet();
    $MockedUIBasketOrder->MakePurchase($MockedServlet);
    ok( !WasCalled( $MockedUIBasketOrder, '_HandleSuccessfulPurchaseResponse' ), "$SubTestName - _HandleSuccessfulPurchaseResponse has not been called." );
    ok( !WasCalled( $MockedUIBasketOrder, '_SaveTransactionNumber' ), "$SubTestName - _SaveTransactionNumber has not been called." );
    ok( WasCalled( $MockedUIBasketOrder, '_writeDataToOutput' ), "$SubTestName - _writeDataToOutput has not been called." );
    ok( !WasCalled( $MockedUIBasketOrder, '_SaveTransactionToken' ), "$SubTestName - _SaveTransactionToken has not been called." );

    my $aExpectedResponse = [{
        'Error' => {
                'Name' => JSON_FAILURE_MESSAGES->{'Name'}->{'Purchase'},
                'Reason' => JSON_FAILURE_MESSAGES->{'Reason'}->{'ConnectionError'}
        }
    }];
    my $aActualResponse = GetParametersFromMockifyCall($MockedUIBasketOrder, '_writeDataToOutput');
    is_deeply($aActualResponse, $aExpectedResponse ,"$SubTestName - _writeDataToOutput has correct parameters." );

    return;
}

sub _test_MakePurchase_WithError {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUIBasketOrder = $self->_buildUIBasketOrder_WithError();
    my $MockedServlet       = $self->_buildServlet();

    lives_ok( sub { $MockedUIBasketOrder->MakePurchase($MockedServlet); }, "$SubTestName - Did not Threw a SpreedlyMakePurchaseError exception" );
    ok( WasCalled( $MockedUIBasketOrder, '_HandleCriticalErrorOnRequest' ), "$SubTestName - _HandleCriticalErrorOnRequest has been called." );
    ok( !WasCalled( $MockedUIBasketOrder, '_SaveTransactionToken' ), "$SubTestName - _SaveTransactionToken has not been called." );

    return;
}

sub _test_CompleteTransaction_SuccessfulTransaction_SuccessState_ThreeDSAvailable {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUIBasketOrder = $self->_buildUIBasketOrder_SuccessfulTransactionSuccessState();
    my $MockedServlet       = $self->_buildServlet();

    $MockedUIBasketOrder->CompleteTransaction($MockedServlet);
    ok( WasCalled( $MockedUIBasketOrder, '_SaveTransactionNumber' ), "$SubTestName - _SaveTransactionNumber has been called." );
    ok( WasCalled( $MockedUIBasketOrder, '_writeDataToOutput' ), "$SubTestName - _writeDataToOutput has been called." );
    ok( !WasCalled( $MockedUIBasketOrder, '_SaveTransactionToken' ), "$SubTestName - _SaveTransactionToken has not been called." );

    my $hExpectedTransaction = $self->_getSuccessfulTransaction(); 

    my $aExpectedSaveTransactionParameters = [$hExpectedTransaction, $MockedServlet->object()];
    my $aActualSaveTransactionParameters = GetParametersFromMockifyCall($MockedUIBasketOrder, '_SaveTransactionNumber');
    is_deeply($aActualSaveTransactionParameters, $aExpectedSaveTransactionParameters ,"$SubTestName - _SaveTransactionNumber has correct parameters." );

    my $aExpectedResponse = [{'Transaction' => {
        'state' => $hExpectedTransaction->{'state'},
        'required_action' => $hExpectedTransaction->{'required_action'},
        'challenge_url' => $hExpectedTransaction->{'challenge_url'},
        'device_fingerprint_form' => $hExpectedTransaction->{'device_fingerprint_form'},
    }}];
    my $aActualResponse = GetParametersFromMockifyCall($MockedUIBasketOrder, '_writeDataToOutput');
    is_deeply($aActualResponse, $aExpectedResponse ,"$SubTestName - _writeDataToOutput has correct parameters." );

    return;
}

sub _test_CompleteTransaction_SuccessfulTransaction_PendingState_ThreeDSNotAvailable {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsThreeDSAvailable = 0;
    my $MockedUIBasketOrder = $self->_buildUIBasketOrder_SuccessfulTransactionPendingState($IsThreeDSAvailable);
    my $MockedServlet       = $self->_buildServlet();
    $MockedUIBasketOrder->CompleteTransaction($MockedServlet);
    ok( WasCalled( $MockedUIBasketOrder, '_writeDataToOutput' ), "$SubTestName - _writeDataToOutput has been called." );
    ok( !WasCalled( $MockedUIBasketOrder, '_SaveTransactionNumber' ), "$SubTestName - _SaveTransactionNumber has not been called." );
    ok( !WasCalled( $MockedUIBasketOrder, '_SaveTransactionToken' ), "$SubTestName - _SaveTransactionToken has not been called." );

    my $aExpectedResponse = [{
        'Error' => {
                'Name' => JSON_FAILURE_MESSAGES->{'Name'}->{'Complete'},
                'Reason' => JSON_FAILURE_MESSAGES->{'Reason'}->{'CardDeclined'}
        }
    }];
    my $aActualResponse = GetParametersFromMockifyCall($MockedUIBasketOrder, '_writeDataToOutput');
    is_deeply($aActualResponse, $aExpectedResponse ,"$SubTestName - _writeDataToOutput has correct parameters." );

    return;
}

sub _test_CompleteTransaction_SuccessfulTransaction_PendingState_ThreeDSAvailable {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsThreeDSAvailable = 1;
    my $MockedUIBasketOrder = $self->_buildUIBasketOrder_SuccessfulTransactionPendingState($IsThreeDSAvailable);
    my $MockedServlet       = $self->_buildServlet();
    $MockedUIBasketOrder->CompleteTransaction($MockedServlet);
    ok( WasCalled( $MockedUIBasketOrder, '_writeDataToOutput' ), "$SubTestName - _writeDataToOutput has been called." );
    ok( !WasCalled( $MockedUIBasketOrder, '_SaveTransactionNumber' ), "$SubTestName - _SaveTransactionNumber has not been called." );
    ok( !WasCalled( $MockedUIBasketOrder, '_SaveTransactionToken' ), "$SubTestName - _SaveTransactionToken has not been called." );

    my $hExpectedTransaction = $self->_getPendingTransaction();

    my $aExpectedResponse = [{'Transaction' => {
        'state' => $hExpectedTransaction->{'state'},
        'required_action' => $hExpectedTransaction->{'required_action'},
        'challenge_url' => $hExpectedTransaction->{'challenge_url'},
        'device_fingerprint_form' => $hExpectedTransaction->{'device_fingerprint_form'},
    }}];
    my $aActualResponse = GetParametersFromMockifyCall($MockedUIBasketOrder, '_writeDataToOutput');
    is_deeply($aActualResponse, $aExpectedResponse ,"$SubTestName - _writeDataToOutput has correct parameters." );

    return;
}

sub _test_CompleteTransaction_SuccessfulTransaction_ErrorState {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUIBasketOrder = $self->_buildUIBasketOrder_SuccessfulTransactionErrorState();
    my $MockedServlet       = $self->_buildServlet();
    $MockedUIBasketOrder->CompleteTransaction($MockedServlet);
    ok( WasCalled( $MockedUIBasketOrder, '_writeDataToOutput' ), "$SubTestName - _writeDataToOutput has been called." );
    ok( !WasCalled( $MockedUIBasketOrder, '_SaveTransactionNumber' ), "$SubTestName - _SaveTransactionNumber has not been called." );

    my $hExpectedTransaction = $self->_getErrorTransaction();

     my $aExpectedResponse = [{'Transaction' => {
        'state' => $hExpectedTransaction->{'state'},
        'required_action' => $hExpectedTransaction->{'required_action'},
        'challenge_url' => $hExpectedTransaction->{'challenge_url'},
        'device_fingerprint_form' => $hExpectedTransaction->{'device_fingerprint_form'},
    }}];
    my $aActualResponse = GetParametersFromMockifyCall($MockedUIBasketOrder, '_writeDataToOutput');
    is_deeply($aActualResponse, $aExpectedResponse ,"$SubTestName - _writeDataToOutput has correct parameters." );

    return;
}

sub _test_CompleteTransaction_UnsuccessfulTransaction {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUIBasketOrder = $self->_buildUIBasketOrder_UnsuccessfulTransaction();
    my $MockedServlet       = $self->_buildServlet();
    $MockedUIBasketOrder->CompleteTransaction($MockedServlet);
    ok( !WasCalled( $MockedUIBasketOrder, '_HandleSuccessfulPurchaseResponse' ), "$SubTestName - _HandleSuccessfulPurchaseResponse has not been called." );
    ok( !WasCalled( $MockedUIBasketOrder, '_SaveTransactionNumber' ), "$SubTestName - _SaveTransactionNumber has not been called." );
    ok( WasCalled( $MockedUIBasketOrder, '_writeDataToOutput' ), "$SubTestName - _writeDataToOutput has been called." );

    my $aExpectedResponse = [{
        'Error' => {
                'Name' => JSON_FAILURE_MESSAGES->{'Name'}->{'Complete'},
                'Reason' => JSON_FAILURE_MESSAGES->{'Reason'}->{'CardDeclined'}
        }
    }];
    my $aActualResponse = GetParametersFromMockifyCall($MockedUIBasketOrder, '_writeDataToOutput');
    is_deeply($aActualResponse, $aExpectedResponse ,"$SubTestName - _writeDataToOutput has correct parameters." );

    return;
}

sub _test_CompleteTransaction_WithError {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUIBasketOrder = $self->_buildUIBasketOrder_WithError();
    my $MockedServlet       = $self->_buildServlet();

    lives_ok( sub { $MockedUIBasketOrder->CompleteTransaction($MockedServlet); }, "$SubTestName - Did not Threw a SpreedlyCompleteTransactionError exception" );
    ok( WasCalled( $MockedUIBasketOrder, '_HandleCriticalErrorOnRequest' ), "$SubTestName - _HandleCriticalErrorOnRequest has been called." );

    return;
}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------

sub _buildUIBasketOrder_SuccessfulTransactionSuccessState {
    my $self = shift;

    my $hReturnValues = {
        '_makePurchase' => $self->_getSuccessfulTransaction(),
        '_completeTransaction' => $self->_getSuccessfulTransaction(),
        '_LogPayment'          => undef,
        '_GetPaymentInfoToken' => PAYMENT_INFO_TOKEN,
        '_createCustomerOrder' => $self->_buildCustomerOrder(),
        '_isThreeDSAvailable' => 1,
    };

    my $ThrowsError = 0;

    return $self->_createUIBasketOrder( $hReturnValues, $ThrowsError );
}

sub _buildUIBasketOrder_SuccessfulTransactionPendingState {
    my $self = shift;
    my ($IsThreeDSAvailable) = @_;

    $IsThreeDSAvailable //= 0;

    my $hReturnValues = {
        '_makePurchase' => $self->_getPendingTransaction(),
        '_completeTransaction' => $self->_getPendingTransaction(),
        '_LogPayment'          => undef,
        '_GetPaymentInfoToken' => PAYMENT_INFO_TOKEN,
        '_isThreeDSAvailable' => $IsThreeDSAvailable,
    };

    my $ThrowsError = 0;

    return $self->_createUIBasketOrder( $hReturnValues, $ThrowsError );
}

sub _buildUIBasketOrder_SuccessfulTransactionErrorState {
    my $self = shift;

    my $hReturnValues = {
        '_makePurchase' => $self->_getErrorTransaction(),
        '_completeTransaction' => $self->_getErrorTransaction(),
        '_LogPayment'          => undef,
        '_GetPaymentInfoToken' => PAYMENT_INFO_TOKEN,
        '_createCustomerOrder' => $self->_buildCustomerOrder(),
        '_isThreeDSAvailable' => 1,
    };

    my $ThrowsError = 0;

    return $self->_createUIBasketOrder( $hReturnValues, $ThrowsError );
}

sub _buildUIBasketOrder_UnsuccessfulTransaction {
    my $self = shift;

    my $hReturnValues = {
        '_makePurchase' => {
            'successful'    => '0',
            'transactionNo' => '123434',
        },
        '_completeTransaction' => {
            'successful'    => '0',
            'transactionNo' => '123434',
        },
        '_LogPayment'          => undef,
        '_GetPaymentInfoToken' => PAYMENT_INFO_TOKEN,
        '_createCustomerOrder' => $self->_buildCustomerOrder(),
        '_isThreeDSAvailable' => 1,
    };

    my $ThrowsError = 0;

    return $self->_createUIBasketOrder( $hReturnValues, $ThrowsError );
}

sub _buildUIBasketOrder_WithError {
    my $self = shift;

    my $hReturnValues = {
        '_makePurchase' => {
            'successful'    => '0',
            'transactionNo' => '123434',
        },
        '_completeTransaction' => {
            'successful'    => '0',
            'transactionNo' => '123434',
        },
        '_LogPayment'          => undef,
        '_GetPaymentInfoToken' => PAYMENT_INFO_TOKEN,
        '_createCustomerOrder' => $self->_buildCustomerOrder(),
        '_isThreeDSAvailable' => 1,
    };

    my $ThrowsError = 1;

    return $self->_createUIBasketOrder( $hReturnValues, $ThrowsError );
}

sub _buildServlet {
    my $self             = shift;
    my $PaymentInfoToken = PAYMENT_INFO_TOKEN;

    return $self->_createServlet($PaymentInfoToken);
}

sub _buildCustomerOrder {
    my $self = shift;

    my $hParams = {
        'ID'          => '19999',
        'PaidOn'      => undef,
        'CancelledOn' => undef,
    };

    return $self->_createCustomerOrder($hParams);
}

sub _createUIBasketOrder {
    my $self = shift;
    my ( $hReturnValues, $ThrowsError ) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Spreedly::UI::JSON');

    my $aLogPaymentParams = [ 'any', 'string', 'any' ];

    $Mockify->addMockWithReturnValueAndParameterCheck( '_LogPayment', $hReturnValues->{'_LogPayment'}, $aLogPaymentParams );

    $Mockify->addMock(
        '_makePurchase',
        sub {
            shift;
            if ($ThrowsError) {
                Error('ConnectionError');
            }

            return $hReturnValues->{'_makePurchase'};
        }
    );

    $Mockify->addMock(
        '_completeTransaction',
        sub {
            shift;
            if ($ThrowsError) {
                Error('ConnectionError');
            }

            return $hReturnValues->{'_completeTransaction'};
        }
    );
    my $aRetrieveLineItemPaymentParams = [ { 'object' => 'DE_EPAGES::Order::API::Object::Basket' } ];
    my $aRetrieveLineItemPaymentReturn = [ { 'object' => 'DE_EPAGES::Spreedly::API::Object::LineItemPaymentSpreedly' } ];
    $Mockify->addMockWithReturnValueAndParameterCheck( '_RetrieveLineItemPayment', $aRetrieveLineItemPaymentReturn, $aRetrieveLineItemPaymentParams );

    my $hHandleCriticalErrorReturnValue = {
        'Error' => {
            'Name' => 'Purchase',
            'Reason' => 'ConnectionError'
    }};
    $Mockify->addMockWithReturnValueAndParameterCheck( '_HandleCriticalErrorOnRequest', $hHandleCriticalErrorReturnValue, ['object', 'object', 'string'] );
    $Mockify->addMethodSpyWithParameterCheck( '_HandleSuccessfulPurchaseResponse', ['hashref','object', 'object'] );
    $Mockify->addMockWithReturnValueAndParameterCheck( '_SaveTransactionToken', undef, [ 'string', 'object' ] );
    $Mockify->addMockWithReturnValueAndParameterCheck( '_isThreeDSAvailable', $hReturnValues->{'_isThreeDSAvailable'}, [ 'object' ] );

    $Mockify->addMethodSpyWithParameterCheck( 'RetrievePaymentInfoToken', ['object'] );
    $Mockify->addMethodSpyWithParameterCheck( '_SaveTransactionNumber',   [ 'hashref', 'object' ] );
    $Mockify->addMethodSpyWithParameterCheck( '_SetPaymentAsFailed',      [ 'object' ] );

    $Mockify->addMockWithReturnValueAndParameterCheck( '_writeDataToOutput', undef, ['hashref'] );

    return $Mockify->getMockObject();
}

sub _createBasket {
    my $self        = shift;
    my $hAttributes = {
        'Site'              => undef,
        'LineItemContainer' => $self->_createLineItemContainer(),
        'GUID'              => '56D5521B-08D3-2F44-728A-7F0001012E94',
    };

    my $Basket = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::Basket', $hAttributes );

    return $Basket;
}

sub _createCustomerOrder {
    my $self = shift;
    my ($hParams) = @_;

    my $hAtributes = {
        'PaidOn'            => $hParams->{'PaidOn'},
        'CancelledOn'       => $hParams->{'CancelledOn'},
        'LineItemContainer' => $self->_createLineItemContainer(),
    };
    my $CustomerOrder = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::CustomerOrder', $hAtributes );

    return $CustomerOrder;
}

sub _createForm {
    my $self = shift;

    my $MockedForm = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::WebInterface::API::Form');

    $MockedForm->mock(
        'executeFormError',
        sub {
            shift;
            my ($hFormError) = @_;

            my $ErrorReason = $hFormError->{'Reason'};
            Error($ErrorReason);

            return;
        }
    );

    $MockedForm->mock(
        'value',
        sub {
            shift;

            return 'ViewMultiCheckoutPayment';
        }
    );

    return $MockedForm;
}

sub _createLineItemContainer {
    my $self = shift;

    my $hAttributes = {
        'GrandTotal' => '80',
        'CurrencyID' => 'GBP',
        'Payment'    => $self->_createLineItemPayment(),
    };

    my $MockedLineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes );

    return $MockedLineItemContainer;
}

sub _createLineItemPayment {
    my $self = shift;

    my $hAttributes = { 'TransTime' => 'Feb 23, 2015 11:44:41 AM', };
    my $MockedLineItemPayment = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Spreedly::API::Object::LineItemPaymentSpreedly', $hAttributes );
    $MockedLineItemPayment->mock(
        'updateTransactionStatus',
        sub {
            shift;

            return;
        }
    );

    return $MockedLineItemPayment;
}

sub _createServlet {
    my $self = shift;
    my ($PaymentInfoToken) = @_;

    my $Basket = $self->_createBasket();
    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Basket);
    $MockifyServlet->addMockWithReturnValue( 'form', $self->_createForm() );
    $MockifyServlet->addParam( 'payment_method_token', $PaymentInfoToken );

    return $MockifyServlet->getMockObject();
}

sub _getSuccessfulTransaction {
    return {
        "state" => PURCHASE_RESPONSE_STATE_SUCEEDED,
        "successful" => "1",
        "card_type" => "visa",
        "checkout_form" => undef,
        "checkout_url" => undef,
        "fingerprint" => "fafcbe9010ddaf21988f3c3b2ccb443f604e",
        "paymentMethodType" => "credit_card",
        "required_action" => "device_fingerprint",
        "token" => "Wyj9Nz4AEWtc5gjHLyIzC0FHGdS",
        "transactionNo" => "50",
        "challenge_url" => 'challenge_url',
        "device_fingerprint_form" => 'device_fingerprint_form',
    };
}

sub _getPendingTransaction {
    return {
        "state" => PURCHASE_RESPONSE_STATE_PENDING,
        "successful" => "1",
        "card_type" => "visa",
        "checkout_form" => undef,
        "checkout_url" => undef,
        "fingerprint" => "fafcbe9010ddaf21988f3c3b2ccb443f604e",
        "paymentMethodType" => "credit_card",
        "required_action" => "device_fingerprint",
        "token" => "Wyj9Nz4AEWtc5gjHLyIzC0FHGdS",
        "transactionNo" => "51",
        "challenge_url" => 'challenge_url',
        "device_fingerprint_form" => 'device_fingerprint_form',
    };
}

sub _getErrorTransaction {
    return {
        "state" => 'error',
        "successful" => "1",
        "card_type" => "visa",
        "checkout_form" => undef,
        "checkout_url" => undef,
        "fingerprint" => "fafcbe9010ddaf21988f3c3b2ccb443f604e",
        "paymentMethodType" => "credit_card",
        "required_action" => "none",
        "token" => "Wyj9Nz4AEWtc5gjHLyIzC0FHGdS",
        "transactionNo" => "123434",
        "challenge_url" => 'challenge_url',
        "device_fingerprint_form" => 'device_fingerprint_form',
    };
}

__PACKAGE__->runScript();

1;
